/**
 * Clase encargada de manejar el comportamiento fisico del elastico
 */
public class Elastic extends Spring  {
   private static int id=0;  // Spring identification
   /**
    * Largo en reposo del elastico
    * */
   protected final float restLength; 
   private float stiffness;
   /**
    * extremo enganchable A del elastico
    * */
   protected AttachableElement a_end;
   /**
    * extremo enganchable B del elastico
    * */
   protected AttachableElement b_end;
/**
 * Constructor por Defecto. 
 * Crea el estado del elastico e inicializa el largo inicial (restlength)
 * y constante de elasticidad del elastico
 */
   private Elastic(){   // nobody can create a block without state
      super(id++);
      stiffness = restLength = 0;
   }
   /**
	 * Constructor de la clase 
	 * @param restLength  Largo en reposos del Elastico
	 * @param stiffness   Costante elastica del elastico 
	 */
   public Elastic(float restLength, float stiffness){
      super(id++);
      this.restLength = restLength;
      this.stiffness = stiffness;
      a_end = b_end = null;
   }
   /**  Engancha el elemento dado en la posicion A del elastico
	 * @param ae Elemento enganchable
	 */
   public void attachAend (AttachableElement ae) {
      a_end = ae;
      ae.attachSpring(this);
   }
   /**  Desengancha el elemento existente en la posicion A
	 *  del elastico
	 */
   public void detachAend(){
      if (a_end == null) return;
      a_end.detachSpring(this);
      a_end=null;
   }
   /**  Engancha el elemento dado en la posicion B del elastico
	 * @param ae Elemento enganchable
	 */
   public void attachBend (AttachableElement ae) {
      b_end = ae;
      ae.attachSpring(this);
   }
   /**  Desengancha el elemento existente en la posicion B
	 *  del elastico
	 */
   public void detachBend(){
      if (b_end == null) return;
      b_end.detachSpring(this);
      b_end = null;
   }
   /**  Retorna la posicion A del elastico
	 * @return Vector con  la posicion A del elastico
	 */
   public Vector2D getAendPosition() {
      if (a_end != null)
         return a_end.getPosition();
      if (b_end != null)
         return b_end.getPosition().minus(new Vector2D(0,restLength));
      return new Vector2D();
   }
   /**  Retorna la posicion B del elastico
	 * @return Vector con  la posicion B del elastico
	 */
   public Vector2D getBendPosition() {
      if (b_end != null)
         return b_end.getPosition();
      if (a_end != null)
         return a_end.getPosition().plus(new Vector2D(0,restLength));
      return new Vector2D(0,restLength);
   }
   /**Entrega la posicion del Elastico
    * @return Vector con la posicion del Elastico
    */
   protected Vector2D getVector() {
      return getBendPosition().minus(getAendPosition());
   }
   /**
    * Calcula la fuerza del elastico. El comportamiento es similar a un resorte (ley de hooke),
    * pero si el estiramiento (stretch) es menor que el largo en reposo(restlength),
    * ejercera fuerza igual a cero.
    * @param ae Elemento Enganchable.
    * @return Fuerza del elastico actual.
    */
   public Vector2D getForce(AttachableElement ae) {
      Vector2D force = new Vector2D(); //se inicializa en 0 por constructor Vector2D
      if ((a_end == null) || (b_end == null))
         return force;
      if ((ae != a_end) && (ae != b_end))
         return force;
      Vector2D springVector = getVector();
      double stretch = springVector.module() - restLength;
      //Si el estiramiento es mayor que el largo en reposo
      //entonces obedece ley de hook. Update tarea 3: se corrigio strech
      if(stretch >0)
    	  force = springVector.unitary().times(stretch*stiffness);
      else
    	  //en caso contrario no ejerce fuerza
    	  return force;
      if (ae == a_end) return force;
      return force.times(-1.0);
   }
   
   /**
    * Entrega informacion del elastico
    * @return id del elastico y posicion (x,y)
    */
   public String getDescription() {
      return "Elastic_"+ getId()+": "+ Vector2D.getDescription()+" ,"+ Vector2D.getDescription();
   }
   /**
    * Entrega las coordenadas de la posicion del elastico
    * @return Posicion del elastico (x,y)
    */
   public String getState() {
      String s = getAendPosition() + "," + getBendPosition();
      return s;
   }
   //para poder modificar las ctes elastica
   /**
    * Modifica el valor de la constante elastica del elastico.
    * @param stiffness valor que tomara la constante de elasticidad del elastico
    * 
    */
   public void setStiffness(float stiffness){
	   this.stiffness=stiffness;
   }
}
